import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:dio/dio.dart';
import 'package:mmoo_base/api/address.dart';
import 'package:mmoo_base/config/app_config.dart';
import 'package:mmoo_base/config/net_config.dart';
import 'package:mmoo_base/dao/base_dao.dart';
import 'package:mmoo_base/dao/dao_result.dart';
import 'package:mmoo_base/db/userinfo_db_provider.dart';
import 'package:mmoo_base/entity/admob_config_entity.dart';
import 'package:mmoo_base/entity/anchor_entity.dart';
import 'package:mmoo_base/entity/block_entity.dart';
import 'package:mmoo_base/entity/care_entity.dart';
import 'package:mmoo_base/entity/daily_entity.dart';
import 'package:mmoo_base/entity/faqlist_entity.dart';
import 'package:mmoo_base/entity/history_call_entity.dart';
import 'package:mmoo_base/entity/history_con_entity.dart';
import 'package:mmoo_base/entity/history_match_entity.dart';
import 'package:mmoo_base/entity/init_config_entity.dart';
import 'package:mmoo_base/entity/init_gift_entity.dart';
import 'package:mmoo_base/entity/language_entity.dart';
import 'package:mmoo_base/entity/lucky_entity.dart';
import 'package:mmoo_base/entity/lucky_number_entity.dart';
import 'package:mmoo_base/entity/lucky_slot_entity.dart';
import 'package:mmoo_base/entity/match_entity.dart';
import 'package:mmoo_base/entity/mian_list_entity.dart';
import 'package:mmoo_base/entity/plus_entity.dart';
import 'package:mmoo_base/entity/pop_entity.dart';
import 'package:mmoo_base/entity/recommend_entity.dart';
import 'package:mmoo_base/entity/room_entity.dart';
import 'package:mmoo_base/entity/share_coin_entity.dart';
import 'package:mmoo_base/entity/token_entity.dart';
import 'package:mmoo_base/entity/toplist_entity.dart';
import 'package:mmoo_base/entity/update_entity.dart';
import 'package:mmoo_base/entity/user_entity.dart';
import 'package:mmoo_base/entity/user_status_entity.dart';
import 'package:mmoo_base/entity/video_entity.dart';
import 'package:mmoo_base/entity/video_info_entity.dart';
import 'package:mmoo_base/entity/video_list_entity.dart';
import 'package:mmoo_base/provider/user_provider.dart';
import 'package:sp_util/sp_util.dart';

class MainDao {
  ///初始化用户信息
  static Future<DataResult> initUserInfo(UserProvider userProvider) async {
    ///用户信息
    String? userId = SpUtil.getString(AppConfig.userId);
    dynamic userEntity;
    if (userId != null) {
      var userDbProvider = UserInfoDbProvider();
      userEntity = await userDbProvider.getUserInfo(userId);
      if (userEntity != null) {
        userProvider.setUserEntity(userEntity);
      }
    }
    bool? isLogin = userEntity != null && SpUtil.getBool(AppConfig.userLogin)!;
    return DataResult(userEntity, isLogin == true);
  }

  ///更新本地数据库
  static Future<UserEntity> updateUserInfo(UserEntity userEntity,
      {bool update = false, bool submit = false}) async {
    var userDbProvider = UserInfoDbProvider();
    if (update) {
      var userEntityPro = await userDbProvider.getUserInfo(userEntity.id);
      if (userEntityPro == null) {
        if (userEntity.age == '0001-01-01') {
          userEntity.gender = '-1';
          userEntity.localData = true;
          userEntity.age = null;
          userEntity.country = null;
          userEntity.likes = [];
          userEntity.rechargeCount = 0;
        } else {
          userEntity.localData = false;
        }
      } else {
        userEntity.localData = false;
        userEntity.rechargeCount = userEntityPro.rechargeCount ?? 0;
      }
    }
    userDbProvider.insert(
      userEntity.id,
      jsonEncode(
        userEntity.toJson(),
      ),
    );
    return userEntity;
  }

  ///更新本地数据库-只更新某些数据
  static Future<UserEntity?> updateUserItem(
      String? userId, UserProvider userProvider,
      {bool? localData,
      String? gender,
      String? age,
      String? country,
      List? likes,
      String? name,
      String? sign,
      String? headImage,
      int? rechargeCount}) async {
    var userDbProvider = UserInfoDbProvider();
    var userEntity = await userDbProvider.getUserInfo(userId);
    if (userEntity != null) {
      var change = false;
      if (localData != null) {
        if (userEntity.localData == null || userEntity.localData != localData) {
          userEntity.localData = localData;
          change = true;
        }
      }
      if (gender != null && userEntity.gender != gender) {
        userEntity.gender = gender;
        change = true;
      }
      if (age != null && userEntity.age != age) {
        userEntity.age = age;
        change = true;
      }
      if (country != null && userEntity.country != country) {
        userEntity.country = country;
        change = true;
      }
      if (likes != null) {
        if (userEntity.likes == null ||
            userEntity.likes!.length != likes.length) {
          userEntity.likes = likes;
          change = true;
        } else {
          for (int i = 0; i < likes.length; i++) {
            if (likes[i] != userEntity.likes![i]) {
              userEntity.likes = likes;
              change = true;
              break;
            }
          }
        }
      }
      if (name != null && userEntity.name != name) {
        userEntity.name = name;
        change = true;
      }
      if (headImage != null && userEntity.headImage != headImage) {
        userEntity.headImage = headImage;
        change = true;
      }
      if (rechargeCount != null && userEntity.rechargeCount != rechargeCount) {
        userEntity.rechargeCount = rechargeCount;
        change = true;
      }
      if (change) {
        userDbProvider.insert(
          userEntity.id,
          jsonEncode(
            userEntity.toJson(),
          ),
        );
        userProvider.setUserEntity(userEntity);
      }
    }
    return userEntity;
  }

  ///登录
  static Future<DataResult> login(
      String? userName, String? password, UserProvider userProvider) async {
    String? encode =
        await BaseDao.encryptToBase64(password!, NetConfig.encryptDesPass);
    Map other = {'email': userName, 'pwd': encode};
    var res =
        await BaseDao.fromBaseEncrypt(other, Address.getLogin(), token: false);
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      SpUtil.putString(AppConfig.userId, entity.id!);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(userEntityPro, true);
    }
    return res;
  }

  static Future<DataResult> appleSign(String? user, String? nickname,
      String? userStr, String? email, UserProvider userProvider) async {
    Map other = {
      'user': user,
      'nickname': nickname,
      'userStr': userStr,
      'email': email
    };
    var res =
        await BaseDao.fromBaseJson(other, Address.appleSign(), token: false);
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      SpUtil.putString(AppConfig.userId, entity.id!);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(userEntityPro, true);
    }
    return res;
  }

  static Future<DataResult> googleSign(String? openId, String? name,
      String? headImage, String? email, UserProvider userProvider) async {
    Map other = {
      'openId': openId,
      'name': name,
      'headImage': headImage,
      'type': '0'
    };
    var res =
        await BaseDao.fromBaseJson(other, Address.googleSign(), token: false);
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      SpUtil.putString(AppConfig.userId, entity.id!);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(userEntityPro, true);
    }
    return res;
  }

  static Future<DataResult> guestSign(UserProvider userProvider) async {
    var res = await BaseDao.fromBaseJson({}, Address.guestSign(), token: false);
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      SpUtil.putString(AppConfig.userId, entity.id!);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(userEntityPro, true);
    }
    return res;
  }

  ///注册
  static Future<DataResult> signUp(
      String? userEmail, String? password, UserProvider userProvider) async {
    String? encode =
        await BaseDao.encryptToBase64(password!, NetConfig.encryptDesPass);
    Map other = {'email': userEmail, 'pwd': encode};
    var res =
        await BaseDao.fromBaseEncrypt(other, Address.signUp(), token: false);
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      SpUtil.putString(AppConfig.userId, entity.id!);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      var userEntity = UserEntity();
      userEntity.id = entity.id;
      userEntity.token = entity.token ?? '';
      userEntity.localData = true;
      userEntity.email = userEmail;
      userEntity.gender = '-1';
      userEntity.likes = [];
      userEntity.rechargeCount = 0;
      updateUserInfo(userEntity);
      userProvider.setUserEntity(userEntity);
      return DataResult(entity, true);
    }
    return res;
  }

  ///完善资料
  static Future<DataResult> regUserInfo(
      String? gender, String? birthday, String? name, UserProvider userProvider,
      {String? inviteCode}) async {
    Map other = {
      'gender': gender,
      'birthday': birthday,
      'name': name,
      'invCode': inviteCode ?? ''
    };
    var res = await BaseDao.fromBaseJson(other, Address.regUserInfo());
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      UserEntity userEntityPro =
          await updateUserInfo(entity, update: true, submit: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(entity, true);
    }
    return res;
  }

  ///提交标签
  static Future<DataResult> setMatchLabel(List<String> likes) async {
    Map other = {'anchorLabel': likes, 'talkLabel': []};
    var res = await BaseDao.fromBaseEncrypt(other, Address.setMatchLabel());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取自己用户信息
  static Future<DataResult> getUserInfo(UserProvider userProvider) async {
    // DataResult vipRes = await OtherDao.getPlusVip();
    var res = await BaseDao.fromBaseEncrypt({}, Address.getUserInfo());
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      // if (vipRes.result) {
      //   userEntityPro.isVip = vipRes.data.level;
      // }
      userProvider.setUserEntity(userEntityPro);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取自己用户信息
  static Future<DataResult> hideLogin(UserProvider userProvider) async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.hideLogin());
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      Address.baseToken = entity.token ?? '';
      SpUtil.putString(AppConfig.appToken, entity.token!);
      UserEntity userEntityPro = await updateUserInfo(entity, update: true);
      userProvider.setUserEntity(userEntityPro);
      return DataResult(entity, true);
    }
    return res;
  }

  ///保存用户信息
  static Future<DataResult> saveUserInfo(
      String? nickname,
      String? signature,
      List<UserPic>? pic,
      String? birthday,
      String? country,
      String? gender) async {
    Map other = {
      'nickname': nickname,
      'signature': signature,
      'pic': pic,
      'birthday': birthday,
      'country': country,
      'gender': gender
    };
    if (pic?.isEmpty == false) {
      other['headImage'] = pic?.first.url;
    }
    var res = await BaseDao.fromBaseEncrypt(other, Address.saveUserInfo());
    if (res.result) {
      var entity = UserEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///检测原密码
  static Future<DataResult> checkPwd(String? password) async {
    String? encode =
        await BaseDao.encryptToBase64(password!, NetConfig.encryptDesPass);
    Map other = {'pwd': encode};
    var res = await BaseDao.fromBaseJson(other, Address.checkPwd());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///修改密码
  static Future<DataResult> updateUserPassword(
      String? oldPwd, String? newPwd) async {
    String? encode =
        await BaseDao.encryptToBase64(oldPwd!, NetConfig.encryptDesPass);
    String? encodeNew =
        await BaseDao.encryptToBase64(newPwd!, NetConfig.encryptDesPass);
    Map other = {'oldPwd': encode, 'newPwd': encodeNew};
    var res = await BaseDao.fromBaseJson(other, Address.updateUserPassword());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///发送验证码
  static Future<DataResult> sendVerifyCode(String? email) async {
    Map other = {'email': email};
    var res = await BaseDao.fromBaseJson(other, Address.sendVerifyCode());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///验证验证码
  static Future<DataResult> verifyCodeCheck(String? email, String? code) async {
    Map other = {'email': email, 'verifyCode': code};
    var res = await BaseDao.fromBaseJson(other, Address.verifyCodeCheck());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///忘记密码
  static Future<DataResult> forgetPassword(
      String? email, String? password, String? token) async {
    String? encode =
        await BaseDao.encryptToBase64(password!, NetConfig.encryptDesPass);
    Map other = {'email': email, 'newPwd': encode, 'token': token};
    var res = await BaseDao.fromBaseJson(other, Address.forgetPassword());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取列表
  static Future<DataResult> getList(
      int type, int page, int languageId, int size) async {
    Map other = {
      'type': type,
      'num': page,
      'size': size,
      'language': languageId != 0 ? [languageId] : [],
      'isAnchor': 1,
      'sortFlag': 1
    };
    var res = await BaseDao.fromBaseEncryptV3(other, Address.getListV3());
    if (res.result) {
      var entity = MainListEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取列表-排行榜
  static Future<DataResult> getLeaderList(int type, int day,
      [int? mutiNum, int? totalNum]) async {
    Map other = {
      'type': type,
      'day': day,
      'mutiNum': mutiNum,
      'totalNum': totalNum
    };
    var res = await BaseDao.fromBaseJson(other, Address.getLeaderList());
    if (res.result) {
      var entity = TopListEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取列表-排行榜-我的排名-金主榜
  static Future<DataResult> getMyCost(int type) async {
    Map other = {'type': type};
    var res = await BaseDao.fromBaseJson(other, Address.getMyCost());
    if (res.result) {
      var entity = MatchCostUser.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取列表-排行榜详情
  static Future<DataResult> getLoverTopList(String? anchorId, int type) async {
    Map other = {'anchorId': anchorId, 'type': type};
    var res = await BaseDao.fromBaseJson(other, Address.getLoverTopList());
    if (res.result) {
      var entity = TopListEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///设置FCM token
  static Future<DataResult> setFirebaseToken(String? token) async {
    Map other = {'token': token};
    var res = await BaseDao.fromBaseJson(other, Address.setFirebaseToken());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取Room列表
  static Future<DataResult> getRoomList(
      String? language, String? label, int page,
      {String? type}) async {
    Map other = {
      'language': language,
      'label': label,
      'pageIndex': page,
      'type': type
    };
    var res = await BaseDao.fromBaseEncrypt(other, Address.getRoomList());
    if (res.result) {
      var entity = RoomEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取RoomInfo
  static Future<DataResult> getRoomInfo() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.getRoomInfo());
    if (res.result) {
      var entity = RoomList.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///Room新建
  static Future<DataResult> setRoomCreate(String? cover, String? roomName,
      String? language, String? label, String? liveNotice) async {
    Map other = {
      'cover': cover,
      'roomName': roomName,
      'language': language,
      'label': label,
      'liveNotice': liveNotice
    };
    var res = await BaseDao.fromBaseEncrypt(other, Address.setRoomCreate());
    if (res.result) {
      var entity = RoomList.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///Room编辑
  static Future<DataResult> setRoomEdit(
      int? id,
      String? cover,
      String? roomName,
      String? language,
      String? label,
      String? liveNotice) async {
    Map other = {
      'id': id,
      'cover': cover,
      'roomName': roomName,
      'language': language,
      'label': label,
      'liveNotice': liveNotice
    };
    var res = await BaseDao.fromBaseEncrypt(other, Address.setRoomEdit());
    if (res.result) {
      var entity = RoomList.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///Room删除
  static Future<DataResult> setRoomDelete(int id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseEncrypt(other, Address.setRoomDelete());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///Room关注/取消
  static Future<DataResult> setRoomFollow(int? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseEncrypt(other, Address.setRoomFollow());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  //获取个人介绍视频
  static Future<DataResult> getAnchorVideoInfo(String? anchorUserId) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getAnchorVideoInfo(),
        post: false, queryParameters: {'anchorUserId': anchorUserId});
    if (res.result) {
      var entity = VideoInfoEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  //获取推荐主播
  static Future<DataResult> getRecommendAnchorList(
      int mutiNum, int total) async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getRecommendAnchorList(),
        post: false, queryParameters: {'mutiNum': mutiNum, 'total': total});
    if (res.result) {
      var data =
          (res.data as List).map((v) => RecommendEntity.fromJson(v)).toList();
      return DataResult(data, true);
    }
    return res;
  }

  ///上传头像
  static Future<DataResult> upLoadHeadImage(String? headImage) async {
    String? image = await BaseDao.compressImage(headImage!);
    var name = image.substring(image.lastIndexOf('/') + 1, image.length);
    FormData formData = FormData.fromMap(
        {'file': MultipartFile.fromFileSync(image, filename: name)});
    var res = await BaseDao.fromBaseJson(formData, Address.upLoadHeadImage());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///上传图片
  static Future<DataResult> upLoadImage(List<UserPic> list) async {
    int position = -1;
    next() async {
      position++;
      String? imageUrl = list[position].url;
      if (list[position].picId?.isEmpty == false) {
        list[position].id = list[position].picId;
      }
      if (imageUrl?.startsWith('http') == false) {
        String? image = await BaseDao.compressImage(imageUrl!);
        var name = image.substring(image.lastIndexOf('/') + 1, image.length);
        FormData formData = FormData.fromMap(
            {'file': MultipartFile.fromFileSync(image, filename: name)});
        var res = await BaseDao.fromBaseJson(formData, Address.upLoadImage());
        if (res.result) {
          var entity = TokenEntity.fromJson(res.data);
          list[position].id = entity.id;
          list[position].picId = entity.id;
          list[position].url = entity.imageUrl;
          if (position != list.length - 1) return await next();
          return DataResult(list, true);
        }
        return res;
      } else if (position != list.length - 1) {
        return await next();
      }
      return DataResult(list, true);
    }

    return await next();
  }

  static Future<DataResult> upLoadImageString(List<String> list,
      [bool id = true]) async {
    int position = -1;
    List<String> listId = [];
    next() async {
      position++;
      String? imageUrl = list[position];
      if (imageUrl.startsWith('http') == false) {
        String? image = await BaseDao.compressImage(imageUrl);
        var name = image.substring(image.lastIndexOf('/') + 1, image.length);
        FormData formData = FormData.fromMap(
            {'file': MultipartFile.fromFileSync(image, filename: name)});
        var res = await BaseDao.fromBaseJson(formData, Address.upLoadImage());
        if (res.result) {
          var entity = TokenEntity.fromJson(res.data);
          if (id) {
            listId.add(entity.id ?? '');
          } else {
            listId.add(entity.imageUrl ?? '');
          }
          if (position != list.length - 1) return await next();
          return DataResult(listId, true);
        }
        return res;
      } else if (position != list.length - 1) {
        return await next();
      }
      return DataResult(listId, true);
    }

    return await next();
  }

  ///上传日志
  static Future<DataResult> upLoadLog(String? userId, String file) async {
    var name = file.substring(file.lastIndexOf('/') + 1, file.length);
    FormData formData = FormData.fromMap({
      'file': MultipartFile.fromFileSync(file, filename: name),
      'userId': userId,
    });
    var res = await BaseDao.fromBaseJson(formData, Address.upLoadLog());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///检查更新
  static Future<DataResult> checkUpdate() async {
    var res = await BaseDao.fromBaseJson({}, Address.checkUpdate());
    if (res.result) {
      dynamic entity;
      if (res.data != null) {
        entity = UpdateEntity.fromJson(res.data);
      }
      return DataResult(entity, true);
    }
    return res;
  }

  ///配置数据
  static Future<DataResult> initConfig(List<String> configs) async {
    var res = await BaseDao.fromBaseEncrypt(
        {'configs': configs}, Address.initConfig(),
        token: false);
    if (res.result) {
      var entity = InitConfigEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///配置数据-新礼物列表
  static Future<DataResult> initGift() async {
    var res = await BaseDao.fromBaseJson({}, Address.initGift(), token: false);
    if (res.result) {
      var entity = InitGiftEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取消费历史列表
  static Future<DataResult> historyConList(int page) async {
    Map other = {'pageIndex': page};
    var res = await BaseDao.fromBaseEncrypt(other, Address.historyConList());
    if (res.result) {
      var entity = HistoryConEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取历史列表
  static Future<DataResult> historyCallList(int page) async {
    Map other = {'pageIndex': page};
    var res = await BaseDao.fromBaseEncrypt(other, Address.historyCallList());
    if (res.result) {
      var entity = HistoryCallEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取match列表
  static Future<DataResult> historyMatchList(int page) async {
    Map other = {'pageIndex': page};
    var res = await BaseDao.fromBaseEncrypt(other, Address.historyMatchList());
    if (res.result) {
      var entity = HistoryMatchEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取关注列表
  static Future<DataResult> careList(int type, int page) async {
    Map other = {'type': type, 'page': page};
    var res = await BaseDao.fromBaseJson(other, Address.careList());
    if (res.result) {
      var entity = CareEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取黑名单列表
  static Future<DataResult> blockList(int page) async {
    Map other = {'pageIndex': page};
    var res = await BaseDao.fromBaseJson(other, Address.blockList());
    if (res.result) {
      var entity = BlockEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///移除黑名单列表
  static Future<DataResult> removeBlock(String? userId) async {
    Map other = {'userId': userId};
    var res = await BaseDao.fromBaseJson(other, Address.removeBlock());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb信息
  static Future<DataResult> getOtherUserInfo(String? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseEncrypt(other, Address.getOtherUserInfo());
    if (res.result) {
      var entity = AnchorEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb信息
  static Future<DataResult> anchorInfo(String? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseEncrypt(other, Address.anchorInfo());
    if (res.result) {
      var entity = AnchorEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb信息
  static Future<DataResult> anchorInfoV3(int anchorUserId) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.anchorInfoV3(),
        post: false, queryParameters: {'anchorUserId': anchorUserId});
    if (res.result) {
      var entity = AnchorEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb小视频信息
  static Future<DataResult> videoListInfo(int page, String? userId) async {
    Map other = {'pageIndex': page, 'userId': userId};
    var res = await BaseDao.fromBaseJson(other, Address.videoListInfo());
    if (res.result) {
      var entity = VideoListEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb购买小视频信息
  static Future<DataResult> buyVideo(String? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseJson(other, Address.buyVideo());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb喜欢小视频信息
  static Future<DataResult> favVideo(String? id, String? type) async {
    Map other = {'videoId': id, 'type': type};
    var res = await BaseDao.fromBaseJson(other, Address.favVideo());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取zb查看小视频信息
  static Future<DataResult> lookVideo(String? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseJson(other, Address.lookVideo());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> anchorSeeInfo(String? id) async {
    Map other = {'anchorUserId': id, 'eventId': '0'};
    var res = await BaseDao.fromBaseJson(other, Address.anchorSeeInfo());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取视频详细信息
  static Future<DataResult> videoInfo(String? videoId) async {
    Map other = {'videoId': videoId};
    var res = await BaseDao.fromBaseJson(other, Address.videoInfo());
    if (res.result) {
      var entity = VideoEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取用户信息-在线状态和名字
  static Future<DataResult> getUserOnline(String? id) async {
    Map other = {'id': id};
    var res = await BaseDao.fromBaseEncrypt(other, Address.getUserOnline());
    if (res.result) {
      var entity = UserOnline.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取用户信息-在线状态和名字-批量
  static Future<DataResult> getUserOnlineList(List<String> ids) async {
    Map other = {'ids': ids};
    var res = await BaseDao.fromBaseEncrypt(other, Address.getUserOnlineList());
    if (res.result) {
      var entity = UserStatusEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///ppeizb
  static Future<DataResult> matchAnchor() async {
    Map other = {'type': '1'};
    var res = await BaseDao.fromBaseEncrypt(other, Address.matchAnchor());
    if (res.result) {
      var entity = AnchorEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> matchAnchorV2() async {
    var res = await BaseDao.fromBaseJson({}, Address.matchAnchorV2());
    if (res.result) {
      var entity = MatchEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> matchUnLike(String? id) async {
    Map other = {'userId': id};
    var res = await BaseDao.fromBaseJson(other, Address.matchUnLike());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///zb关注
  static Future<DataResult> anchorCare(String? id) async {
    Map other = {'userId': id};
    var res = await BaseDao.fromBaseJson(other, Address.anchorCare());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///zb举报
  static Future<DataResult> anchorReport(
      String? id, String? content, String? block) async {
    Map other = {'id': id, 'block': block, 'content': content ?? ''};
    var res = await BaseDao.fromBaseJson(other, Address.anchorReport());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///发送消息
  static Future<DataResult> sendMessage(String? anchorId) async {
    Map other = {'anchorId': anchorId};
    var res = await BaseDao.fromBaseEncrypt(other, Address.sendMessage());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///送礼物
  static Future<DataResult> sendGift(String? giftId, String? anchorId,
      {int? type}) async {
    Map other = {'giftId': giftId, 'anchorId': anchorId, 'type': type};
    var res = await BaseDao.fromBaseJson(other, Address.sendGift());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///送礼物-花钱
  static Future<DataResult> sendGiftTemp(String? price) async {
    Map other = {'price': price};
    var res = await BaseDao.fromBaseJson(other, Address.sendGiftTemp());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///通话前置检测
  static Future<DataResult> callBefore(
      String? anchorId, String? callId, int? type) async {
    Map other = {'userId': anchorId, 'callId': callId, 'callType': type};
    var res = await BaseDao.fromBaseEncrypt(other, Address.callBefore());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///通话评价
  static Future<DataResult> callComment(String? anchorId, String? callId,
      String? isUp, List<String> label) async {
    Map other = {
      'anchorId': anchorId,
      'callId': callId,
      'isUp': isUp,
      'label': label
    };
    var res = await BaseDao.fromBaseEncrypt(other, Address.callComment());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///删除账号
  static Future<DataResult> delUser(String? type, String? reason) async {
    Map other = {'type': type, 'reason': reason ?? ''};
    var res = await BaseDao.fromBaseJson(other, Address.delUser());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///反馈
  static Future<DataResult> feedBack(
      int type, String? suggest, List<String> picList, String? email) async {
    Map other = {
      'type': type,
      'suggest': suggest ?? '',
      'picList': picList,
      'email': email ?? ''
    };
    var res = await BaseDao.fromBaseJson(other, Address.feedBack());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///v2
  ///获取举报状态
  static Future<DataResult> getReportUser(String? userId) async {
    Map other = {'userId': userId};
    var res = await BaseDao.fromBaseEncrypt(other, Address.getReportUser());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///反馈zb不在线
  static Future<DataResult> userNotOnline(String? userId) async {
    Map other = {'userId': userId};
    var res = await BaseDao.fromBaseEncrypt(other, Address.userNotOnline());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///最近match的用户
  static Future<DataResult> matchLastOne() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.matchLastOne());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///pop列表
  static Future<DataResult> popupList() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.popupList());
    if (res.result) {
      var entity = PopupEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///匹配信息
  static Future<DataResult> matchInfo() async {
    var res = await BaseDao.fromBaseJson({}, Address.matchInfo());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true, time: res.time);
    }
    return res;
  }

  ///签到信息
  static Future<DataResult> dailyInfo() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.dailyInfo());
    if (res.result) {
      var entity = DailyEntity.fromJson(res.data);
      return DataResult(entity, true, time: res.time);
    }
    return res;
  }

  ///签到
  static Future<DataResult> checkDaily() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.checkDaily());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取服务器时间
  static Future<DataResult> timeSync() async {
    var res = await BaseDao.fromBaseJson({}, Address.timeSync());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取vip信息
  static Future<DataResult> getPlusVip() async {
    var res = await BaseDao.fromBaseJson({}, Address.getPlusVip());
    if (res.result) {
      var entity = PlusBean.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  ///获取vip充值列表
  static Future<DataResult> plusVipList() async {
    var res = await BaseDao.fromBaseJson({}, Address.plusVipList());
    if (res.result) {
      var entity = PlusEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> topicUse(String? topicId) async {
    Map other = {'topicId': topicId};
    var res = await BaseDao.fromBaseEncrypt(other, Address.topicUse());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> helpList(String? type) async {
    Map other = {'type': type};
    var res = await BaseDao.fromBaseJson(other, Address.helpList());
    if (res.result) {
      var entity = FaqListEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> isLuckyOpen() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.isLuckyOpen());
    if (res.result) {
      var entity = LuckyBean.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> luckyList() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.luckyList());
    if (res.result) {
      var entity = LuckyEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getLucky() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.getLucky());
    if (res.result) {
      var entity = LuckyModel.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getLuckyNumber(int? number) async {
    Map other = {'betsNumber': '$number'};
    var res = await BaseDao.fromBaseJson(other, Address.getLuckyNumber());
    if (res.result) {
      var entity = LuckyNumberEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getSlotRecordList() async {
    var res = await BaseDao.fromBaseJson({}, Address.getSlotRecordList());
    if (res.result) {
      var entity = LuckySlotEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getLanguagesList() async {
    var res = await BaseDao.fromBaseJson({}, Address.getLanguagesList());
    if (res.result) {
      var entity = LanguageEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> admobConfig() async {
    var res = await BaseDao.fromBaseJson({}, Address.admobConfig());
    if (res.result) {
      var entity = AdmobConfigEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> applovinAddCoin(String? adUnit) async {
    Map other = {'adUnit': adUnit};
    var res = await BaseDao.fromBaseJson(other, Address.applovinAddCoin());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> addCoinByShare(int type, String? param) async {
    Map other = {'type': type, 'param': param};
    var res = await BaseDao.fromBaseEncrypt(other, Address.addCoinByShare());
    if (res.result) {
      var entity = TokenEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getCountShareCoin() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.getCountShareCoin());
    if (res.result) {
      var entity = ShareCoinEntity.fromJson(res.data);
      return DataResult(entity, true);
    }
    return res;
  }

  static Future<DataResult> getRtmToken() async {
    var res =
        await BaseDao.fromBaseEncryptV3({}, Address.getRtmToken(), post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  static Future<DataResult> getRtcToken(String? houseId) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getRtcToken(),
        post: false, queryParameters: {'houseId': houseId});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //创建直播间v3接口
  static Future<DataResult> livePostCreateLiveImpl(
      {required String imageUrl,
      required String language,
      required String liveNotice,
      required String title,
      required int type,
      required int price,
      required int videoPlaceType,
      required int videoPrice,
      required int voicePlaceType,
      required int voicePrice}) async {
    Map<String, dynamic> params = {
      'imageUrl': imageUrl,
      'language': language,
      'liveNotice': liveNotice,
      'title': title,
      'type': type,
      'price': price,
      'videoPlaceType': videoPlaceType,
      'videoPrice': videoPrice,
      'voicePlaceType': voicePlaceType,
      'voicePrice': voicePrice,
    };
    var res = await BaseDao.fromBaseEncryptV3(
      params,
      Address.livePostCreate(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//房主停止直播
  static Future<DataResult> livePostStopLive({required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {"roomId": roomId},
      Address.livePostStopLive(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//主播（不）同意上麦
  static Future<DataResult> livePostAgreeForMike(
      {required int roomId, required int type, required int userId}) async {
    Map<String, dynamic> params = {
      'roomId': roomId,
      'type': type,
      'userId': userId,
    };
    var res = await BaseDao.fromBaseEncryptV3(
      params,
      Address.livePostAgreeForMike(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//申请上麦
  static Future<DataResult> livePostApplyForMike(
      {required int roomId, required int type}) async {
    Map<String, dynamic> params = {
      'roomId': roomId,
      'type': type,
    };
    var res = await BaseDao.fromBaseEncryptV3(
      params,
      Address.livePostApplyForMike(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //取消申请上麦
  static Future<DataResult> livePostCancelMike({required int roomId}) async {
    Map<String, dynamic> params = {
      'roomId': roomId,
    };
    var res = await BaseDao.fromBaseEncryptV3(
      params,
      Address.livePostCancelMike(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //房主下麦
  static Future<DataResult> livePostUserDownMike(
      {required int roomPlaceId, required int roomId}) async {
    var res =
        await BaseDao.fromBaseEncryptV3({}, Address.livePostUserDownMike(),
            queryParameters: {
              'roomId': roomId,
              'roomPlaceId': roomPlaceId,
            });
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //退出房间
  static Future<DataResult> livePostExitRoom(
      {required int roomId, int? roomPlaceId, int? roomUserId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
        'roomPlaceId': roomPlaceId ?? '0',
        'roomUserId': roomUserId ?? '0'
      },
      Address.livePostExitRoom(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//进入房间
  static Future<DataResult> livePostJoinRoom({required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
      },
      Address.livePostJoinRoom(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //加入音频、视频连线(申请：主播同意后上麦，邀请：用户接受邀请上麦)
  static Future<DataResult> livePostUserOnMike(
      {required int roomId, required int placeType, required int type}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'placeType': placeType,
        'roomId': roomId,
        'type': type,
      },
      Address.livePostUserOnMike(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //成员列表
  static Future<DataResult> livePostRoomUserList(
      {required int roomId, required int num, int? size}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'num': num, "size": size ?? 20},
      Address.livePostRoomUserList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//(解除)禁群聊
  static Future<DataResult> livePostMuteUser(
      {required bool isMute, required int userId, required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'muteType': isMute ? 0 : 1,
        'roomId': roomId,
        'userId': userId,
      },
      Address.livePostMute(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //(解除)禁语音

  static Future<DataResult> livePostMuteVoiceUser(
      {required bool isMute, required int userId, required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'voiceType': isMute ? 0 : 1,
        'roomId': roomId,
        'userId': userId,
      },
      Address.livePostSetVoice(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //判断用户是否在直播间

  static Future<DataResult> livePostUserIsOnRoom(
      {required int userId, required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
        'userId': userId,
      },
      Address.livePostIsOnRoom(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//获取直播间列表
  static Future<DataResult> livePostRoomList(
      {required int type, required int num, int? size}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'type': type, 'num': num, "size": size ?? 20},
      Address.livePostLiveList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//编辑直播间
  static Future<DataResult> livePostLiveEdit(
      {required String language,
      required String liveNotice,
      required int liveId,
      required int roomId,
      required String title,
      required int videoPrice,
      required int price,
      required int voicePrice}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'language': language,
        'liveNotice': liveNotice,
        'liveId': liveId,
        'roomId': roomId,
        'title': title,
        'videoPrice': videoPrice,
        'price': price,
        'voicePrice': voicePrice
      },
      Address.livePostLiveEdit(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //解锁房间
  static Future<DataResult> livePostUnlockRoom(
      {required int liveId, required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'liveId': liveId, 'roomId': roomId},
      Address.livePostLiveList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//查询主播最近一次直播信息

  static Future<DataResult> liveGetLastRoomInfo() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.liveGetLastRoomInfo(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //直播间详情
  static Future<DataResult> livePostRoomInfo({required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.liveGetRoomInfo(),
        post: false, queryParameters: {'roomId': roomId});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //房主下麦
  static Future<DataResult> liveGetMasterDownMike({required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.liveGetMasterDownMike(),
        post: false, queryParameters: {'roomId': roomId});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //房主上麦
  static Future<DataResult> liveGetOnMike(
      {required int roomId, required int type}) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.liveGetMasterOnMike(),
        post: false, queryParameters: {"roomId": roomId, "type": type});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//修改自己麦状态
  static Future<DataResult> livePostMuteFlushMike(
      {required int status, //1 开启 2 关闭
      required int roomPlaceId,
      required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'status': status,
        'roomPlaceId': roomPlaceId,
        'roomId': roomId,
      },
      Address.livePostFlushMike(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//拒绝连麦
  static Future<DataResult> livePostUserRefuseConnect(
      {required int roomId}) async {
    var res =
        await BaseDao.fromBaseEncryptV3({}, Address.livePostRefuseConnect(),
            post: false,
            queryParameters: {
              'roomId': roomId,
            });
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//邀请上麦
  static Future<DataResult> livePostMasterInvite(
      {required int userId, required int roomId, required int type}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
        'userId': userId,
        'type': type,
      },
      Address.livePostInvite(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//主播操作用户下麦
  static Future<DataResult> liveMasterDownSeat(
      {required int userId,
      required int roomId,
      required int roomPlaceId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
        'userId': userId,
        'roomPlaceId': roomPlaceId,
      },
      Address.livePostDownSeat(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //（取消）关注用户
  static Future<DataResult> livePostCareUser(
      {required int careType, required int userId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'userId': userId,
        'careType': careType,
      },
      Address.livePostCareUser(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//房间内拉黑
  static Future<DataResult> livePostBlockUser(
      {required bool isCancelBlock, //是否取消拉黑 0 拉黑 1 取消拉黑
      required int userId,
      required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'userId': userId, "blockType": isCancelBlock ? 1 : 0},
      Address.livePostBlockUser(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//（取消）关注房间 0 关注 1 取消关注
  static Future<DataResult> livePostCareRoom(
      {required bool isCareType, required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'careType': isCareType ? 0 : 1},
      Address.livePostCareRoom(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //申请队列
  static Future<DataResult> livePostApplyList(
      {required int roomId, required int num, int? size}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'num': num, "size": size ?? 20},
      Address.livePostApplyList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //黑名单列表
  static Future<DataResult> livePostBlockUserList(
      {required int roomId, required int num, int? size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'num': num, "size": size},
      Address.livePostBlockUserList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //关注列表
  static Future<DataResult> livePostCareUserList(
      {required int roomId, required int num, int? size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'roomId': roomId, 'num': num, "size": size},
      Address.livePostCareUserList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//直播间用户详情
  static Future<DataResult> liveGetUserInfo(
      {required int roomId, required int userId}) async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.liveGetUserInfo(),
        post: false, queryParameters: {"roomId": roomId, "userId": userId});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

//直播v3送礼
  static Future<DataResult> livePostSendGift(
      {required int giftId,
      required int num,
      required int roomId,
      required List<int> userIdList}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'giftId': giftId,
        'roomId': roomId,
        'num': num,
        'userIdList': userIdList,
      },
      Address.livePostSendGift(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  //上传图片
  static Future<DataResult> livePostFileUpload(
      {required String headImage}) async {
    String? image = await BaseDao.compressImage(headImage);
    File imageFile = File(headImage);
    Uint8List bytes = imageFile.readAsBytesSync();
    var name = image.substring(image.lastIndexOf('/') + 1, image.length);
    var baseEntity = await BaseDao.fromBaseEncryptV3(
      {"contentLength": bytes.length.toString(), "fileName": name},
      Address.livePostFileUpload(),
    );
    if (baseEntity.result) {
      try {
        Response response;
        Options options = Options(headers: {
          "Content-Type": baseEntity.data['contentType'],
          "Content-Length": bytes.length.toString()
        });
        response = await Dio().put(baseEntity.data['preUrl'],
            data: MultipartFile.fromBytes(bytes).finalize(),
            options: options, onSendProgress: (count, total) {
          AppConfig.printLog(count / total);
        }, onReceiveProgress: (data, to) {});
        if (response.statusCode == 200) {
          var res = await BaseDao.fromBaseEncryptV3(
            {
              "ossName": baseEntity.data['ossName'],
              "preUrl": baseEntity.data['preUrl']
            },
            Address.livePostFileSuccess(),
          );
          if (res.result) {
            return DataResult(res.data, true);
          }
        }
      } catch (e) {
        AppConfig.printLog(e);
      }
    }
    return baseEntity;
  }

  ///用户体系获取相框列表
  static Future<DataResult> getUserSystemHeadList() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getUserSystemHeadList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系设置用户头像框
  static Future<DataResult> setUserSystemHead(
      {required int headFrameId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'headFrameId': headFrameId},
      Address.getUserSystemSetHead(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系解锁用户头像框
  static Future<DataResult> setUserSystemUnlockHead(
      {required int headFrameId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'headFrameId': headFrameId},
      Address.getUserSystemUnlockHead(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系进场特效列表
  static Future<DataResult> getUserSystemIncomeStateList() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getUserSystemIncomeStateList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系设置进场特效
  static Future<DataResult> postUserSystemSetIncomeState(
      {required int incomeStateId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'incomeStateId': incomeStateId},
      Address.postUserSystemSetIncomeState(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系解锁进场特效
  static Future<DataResult> postUserSystemUnlockIncomeState(
      {required int incomeStateId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'incomeStateId': incomeStateId},
      Address.postUserSystemUnlockIncomeState(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系成就列表
  static Future<DataResult> getUserSystemAchieveList() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getUserSystemAchieveList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系获取可设置成就列表
  static Future<DataResult> postUserSystemAchieveShowList() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.postUserSystemAchieveShowList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系设置成就详情
  static Future<DataResult> postUserSystemAchieveInfo(
      {required int serialNum, required String type}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'serialNum': serialNum,
        'type': type,
      },
      Address.postUserSystemAchieveInfo(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系设置成就
  static Future<DataResult> postUserSystemSetAchieve(
      {required int serialNum, required int sort, required String type}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'serialNum': serialNum,
        'sort': sort,
        'type': type,
      },
      Address.postUserSystemSetAchieve(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户体系设置成就
  static Future<DataResult> postUserSystemCancelAchieve(
      {required int serialNum}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'serialNum': serialNum,
      },
      Address.postUserSystemCancelAchieve(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///主播邀请列表
  static Future<DataResult> postAnchorInviteList(
      {required int num, required int roomId, int size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'num': num,
        'roomId': roomId,
        'size': size,
      },
      Address.postAnchorInviteList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///邀请主播
  static Future<DataResult> postAnchorInviteOther(
      {required List ids, required int roomId, required int liveId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'ids': ids,
        'roomId': roomId,
        'liveId': liveId,
      },
      Address.postAnchorInviteOther(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///（不）同意邀请
  static Future<DataResult> postAnchorAnswerInvite({required int type}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'type': type,
      },
      Address.postAnchorAnswerInvite(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///（获取用户成就信息
  static Future<DataResult> getUserAchieves() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getUserAchieves(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///用户加入直播间成功
  static Future<DataResult> postJoinChatRoom({required int roomId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'roomId': roomId,
      },
      Address.joinChatroom(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 每日经验值详情
  static Future<DataResult> getUserLevelExpList() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getUserLevelExpList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 等级列表
  static Future<DataResult> getUserLevelList() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getUserLevelList(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 解锁等级奖励
  static Future<DataResult> userLevelUnlockLevel(
      {required String level}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'level': level,
      },
      Address.userLevelUnlockLevel(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 积分兑换
  static Future<DataResult> userIntegralExchange(
      {required int convertId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'convertId': convertId,
      },
      Address.userIntegralExchange(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 积分兑换列表
  static Future<DataResult> userIntegralExchangeList(
      {required int num, int size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'num': num,
        'size': size,
      },
      Address.userIntegralExchangeList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 用户积分
  static Future<DataResult> getUserIntegralInfo() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getUserIntegralInfo(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 用户积分抽奖
  static Future<DataResult> getUserIntegralLottery() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getUserIntegralLottery(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 获取抽奖列表
  static Future<DataResult> userIntegralLuckyList() async {
    var res = await BaseDao.fromBaseEncryptV3(
      {},
      Address.userIntegralLuckyList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 积分兑换历史
  static Future<DataResult> userPointExchangeHistoryList(
      {required int num, int size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'num': num,
        'size': size,
      },
      Address.userPointExchangeHistoryList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 积分抽奖历史
  static Future<DataResult> userPointLotteryHistoryList(
      {required int num, int size = 20}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {
        'num': num,
        'size': size,
      },
      Address.userPointLotteryHistoryList(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 日任务
  static Future<DataResult> getTaskDailyTask() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskDailyTask(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 月历史记录
  static Future<DataResult> getTaskMonthHistory() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskMonthHistory(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 月任务
  static Future<DataResult> getTaskMonthTask() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskMonthTask(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 领取奖励
  static Future<DataResult> getTaskReceiveAward() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskReceiveAward(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 周任务
  static Future<DataResult> getTaskWeekTask() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskWeekTask(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 任务
  static Future<DataResult> getTaskGetTask() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getTaskGetTask(),
        queryParameters: {}, post: false);
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 领取任务
  static Future<DataResult> taskReceiveTask({required int taskId}) async {
    var res = await BaseDao.fromBaseEncryptV3(
      {'taskId': taskId},
      Address.taskReceiveTask(),
    );
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 获取积分规则说明
  static Future<DataResult> getUserPointIntegralRule() async {
    var res = await BaseDao.fromBaseEncryptV3(
        {}, Address.getUserPointIntegralRule(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  /// 分享成功上报
  static Future<DataResult> getUserShareSuccess() async {
    var res = await BaseDao.fromBaseEncryptV3({}, Address.getUserShareSuccess(),
        post: false, queryParameters: {});
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }

  ///获取免费消息
  static Future<DataResult> freeMessageNum() async {
    var res = await BaseDao.fromBaseEncrypt({}, Address.freeMessageNum());
    if (res.result) {
      return DataResult(res.data, true);
    }
    return res;
  }
}
