import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:mmoo_base/db/sql_provider.dart';
import 'package:mmoo_base/entity/user_entity.dart';
import 'package:mmoo_base/utils/code_utils.dart';
import 'package:sqflite/sqflite.dart';

///用户表
class UserInfoDbProvider extends BaseDbProvider {
  final String name = 'UserInfoDb';

  final String columnId = '_id';
  final String columnUserId = 'userId';
  final String columnData = 'data';

  String? userId;
  String? data;

  UserInfoDbProvider();

  Map<String, dynamic> toMap(String? userId, String data) {
    Map<String, dynamic> map = {columnUserId: userId, columnData: data};
    return map;
  }

  UserInfoDbProvider.fromMap(Map map) {
    userId = map[columnUserId];
    data = map[columnData];
  }

  @override
  tableSqlString() {
    return tableBaseString(name, columnId) +
        '''
        $columnUserId text not null,
        $columnData text not null)
      ''';
  }

  @override
  tableName() {
    return name;
  }

  Future _getUserProvider(Database db, String? userId) async {
    List<Map<String, dynamic>> maps = await db.query(name,
        columns: [columnId, columnUserId, columnData],
        where: '$columnUserId = ?',
        whereArgs: [userId]);
    if (maps.isNotEmpty) {
      UserInfoDbProvider userProvider = UserInfoDbProvider.fromMap(maps.first);
      return userProvider;
    }
    return null;
  }

  ///插入到数据库
  Future insert(String? userId, String eventMapString) async {
    Database db = await getDataBase();
    var userProvider = await _getUserProvider(db, userId);
    if (userProvider != null) {
      return await db.update(name, toMap(userId, eventMapString),
          where: '$columnUserId = ?', whereArgs: [userId]);
    }
    return await db.insert(name, toMap(userId, eventMapString));
  }

  ///获取事件数据
  Future<UserEntity?> getUserInfo(String? userId) async {
    Database db = await getDataBase();
    var userProvider = await _getUserProvider(db, userId);
    if (userProvider != null) {
      ///使用 compute 的 Isolate 优化 json decode
      var mapData =
          await compute(CodeUtils.decodeMapResult, userProvider.data as String);
      return UserEntity.fromJson(mapData);
    }
    return null;
  }
}
