import 'package:flutter/material.dart';

///底部弹出到中间，控制时间和顺序
class BottomCenterWidget extends StatefulWidget {
  final Widget child;
  final int? index;
  final int duration;
  final double yOffset;

  const BottomCenterWidget(
      {Key? key,
      required this.child,
      this.yOffset = 1.0,
      this.duration = 600,
      this.index})
      : super(key: key);

  @override
  _BottomCenterWidgetState createState() => _BottomCenterWidgetState();
}

class _BottomCenterWidgetState extends State<BottomCenterWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: Duration(milliseconds: widget.duration));
    _animation =
        Tween(begin: Offset(0.0, widget.yOffset), end: const Offset(0.0, 0.0))
            .animate(_controller!);
    Future.delayed(Duration(milliseconds: 200 * widget.index!), () {
      if (!mounted) return;
      _controller?.forward();
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
        position: _animation as Animation<Offset>, child: widget.child);
  }
}
