import 'package:flutter/material.dart';

///底部弹出到顶部，控制时间和顺序
class BottomTopWidget extends StatefulWidget {
  final Widget child;
  final int index;
  final int duration;
  final double start;

  const BottomTopWidget(
      {Key? key,
      required this.child,
      required this.index,
      this.duration = 800,
      this.start = 2.0})
      : super(key: key);

  @override
  _BottomTopWidgetState createState() => _BottomTopWidgetState();
}

class _BottomTopWidgetState extends State<BottomTopWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: Duration(milliseconds: widget.duration));
    _animation =
        Tween(begin: Offset(0.0, widget.start), end: const Offset(0.0, 0.0))
            .animate(_controller!);
    Future.delayed(Duration(milliseconds: 200 * widget.index), () {
      if (!mounted) return;
      _controller?.forward();
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
        position: _animation as Animation<Offset>, child: widget.child);
  }
}
