import 'package:flutter/material.dart';

class Bubble extends StatelessWidget {
  final BubbleDirection direction;
  final Radius? borderRadius;
  final Widget child;
  final Decoration? decoration;
  final EdgeInsetsGeometry? padding;
  final EdgeInsetsGeometry? margin;
  final BoxConstraints? constraints;
  final double? width;
  final double? height;
  final Alignment? alignment;
  final bool? left;
  final double center;
  final double arrowWidth; //箭头宽度
  final double arrowHeight; //箭头高度
  final double minWidth; //内容最小宽度
  final double minHeight; //内容最小高度

  const Bubble({
    Key? key,
    this.direction = BubbleDirection.left,
    required this.child,
    this.decoration,
    this.padding,
    this.margin,
    this.constraints,
    this.width,
    this.height,
    this.alignment,
    this.borderRadius,
    this.left,
    this.center = 2,
    this.arrowWidth = 8.0,
    this.arrowHeight = 16.0,
    this.minWidth = 30.0,
    this.minHeight = 30.0,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ClipPath(
      clipper: _BubbleClipper(
          direction,
          borderRadius ?? const Radius.circular(0),
          left,
          center,
          arrowWidth,
          arrowHeight,
          minWidth,
          minHeight),
      child: Container(
        decoration: decoration,
        child: Container(
          alignment: alignment,
          width: width,
          height: height,
          constraints: constraints ??
              const BoxConstraints()
                  .copyWith(minHeight: minHeight, minWidth: minWidth),
          decoration: decoration,
          padding: padding,
          margin: margin,
          child: child,
        ),
      ),
    );
  }
}

///方向
enum BubbleDirection { left, top, right, bottom }

class _BubbleClipper extends CustomClipper<Path> {
  final BubbleDirection direction;
  final Radius radius;
  final bool? left;
  final double center;
  final double arrowWidth; //箭头宽度
  final double arrowHeight; //箭头高度
  final double minWidth; //内容最小宽度
  final double minHeight; //内容最小高度

  _BubbleClipper(this.direction, this.radius, this.left, this.center,
      this.arrowWidth, this.arrowHeight, this.minWidth, this.minHeight);

  @override
  Path getClip(Size size) {
    final path = Path();
    final path2 = Path();
    final centerPoint = (size.height / 2).clamp(minHeight / 2, minHeight / 2);
    double centerPointWidth = size.width / center;
    if (direction == BubbleDirection.left) {
      //绘制左边三角形
      path.moveTo(0, centerPoint);
      path.lineTo(arrowWidth, centerPoint - arrowHeight / 2);
      path.lineTo(arrowWidth, centerPoint + arrowHeight / 2);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(arrowWidth, 0, (size.width - arrowWidth), size.height),
          radius));
      //合并
      path.addPath(path2, const Offset(0, 0));
    } else if (direction == BubbleDirection.right) {
      //绘制右边三角形
      path.moveTo(size.width, centerPoint);
      path.lineTo(size.width - arrowWidth, centerPoint - arrowHeight / 2);
      path.lineTo(size.width - arrowWidth, centerPoint + arrowHeight / 2);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(0, 0, (size.width - arrowWidth), size.height), radius));
      //合并
      path.addPath(path2, const Offset(0, 0));
    } else if (direction == BubbleDirection.top) {
      //绘制上边三角形
      path.moveTo(centerPointWidth, 0);
      path.lineTo(centerPointWidth - arrowHeight / 2, arrowWidth);
      path.lineTo(centerPointWidth + arrowHeight / 2, arrowWidth);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(
              0, arrowWidth, size.width, size.height + arrowWidth * 2),
          radius));
      //合并
      path.addPath(path2, const Offset(0, 0));
    } else {
      //绘制下边三角形
      if (left == true) {
        centerPointWidth = centerPointWidth / 2;
      } else if (left == false) {
        centerPointWidth = centerPointWidth + centerPointWidth / 2;
      }
      path.moveTo(centerPointWidth, size.height);
      path.lineTo(centerPointWidth - arrowHeight / 2, size.height - arrowWidth);
      path.lineTo(centerPointWidth + arrowHeight / 2, size.height - arrowWidth);
      path.close();
      //绘制矩形
      path2.addRRect(RRect.fromRectAndRadius(
          Rect.fromLTWH(0, 0, size.width, size.height - arrowWidth), radius));
      //合并
      path.addPath(path2, const Offset(0, 0));
    }
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return false;
  }
}
