import 'package:flutter/material.dart';

///隐藏到显示
class FadeScaleWidget extends StatefulWidget {
  final Widget child;

  const FadeScaleWidget(this.child, {Key? key}) : super(key: key);

  @override
  _FadeScaleWidgetState createState() => _FadeScaleWidgetState();
}

class _FadeScaleWidgetState extends State<FadeScaleWidget>
    with TickerProviderStateMixin {
  late AnimationController _controller;
  late Animation<double> _animationFade;
  late Animation<double> _animationScale;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 2000));
    _animationFade = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(parent: _controller, curve: Curves.elasticOut));
    _animationScale = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(parent: _controller, curve: Curves.elasticOut));
    _controller.forward();
    Future.delayed(const Duration(seconds: 10), () {
      if (!mounted) return;
      _controller.reverse();
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FadeTransition(
        opacity: _animationFade,
        child: ScaleTransition(
          scale: _animationScale,
          child: widget.child,
        ));
  }
}
