import 'package:flutter/material.dart';

///隐藏到显示
class FadeWidget extends StatefulWidget {
  final Widget child;
  final int duration;
  final int delayed;
  final bool rest;

  const FadeWidget(this.child,
      {Key? key, this.duration = 1000, this.delayed = 1000, this.rest = false})
      : super(key: key);

  @override
  _FadeWidgetState createState() => _FadeWidgetState();
}

class _FadeWidgetState extends State<FadeWidget> with TickerProviderStateMixin {
  late AnimationController _controllerFade;
  late Animation<double> _animationFade;

  @override
  void initState() {
    _controllerFade = AnimationController(
        vsync: this, duration: Duration(milliseconds: widget.duration));
    if (!widget.rest) {
      _animationFade = Tween(begin: 0.0, end: 1.0).animate(
          CurvedAnimation(parent: _controllerFade, curve: Curves.linear));
    } else {
      _animationFade = Tween(begin: 1.0, end: 0.0).animate(
          CurvedAnimation(parent: _controllerFade, curve: Curves.linear));
    }
    Future.delayed(Duration(milliseconds: widget.delayed), () {
      if (!mounted) return;
      _controllerFade.forward();
    });
    super.initState();
  }

  @override
  void dispose() {
    _controllerFade.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FadeTransition(opacity: _animationFade, child: widget.child);
  }
}
