import 'dart:math';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

///翻转卡片的控件
class FlipWidget extends StatefulWidget {
  final Widget frontChild; //正面
  final Widget backChild; //反面
  final double? indexX;
  final double? indexY;
  final int indexLeft;

  const FlipWidget(
      {Key? key,
      required this.frontChild,
      required this.backChild,
      this.indexX,
      this.indexY,
      this.indexLeft = 0})
      : super(key: key);

  @override
  _FlipWidgetState createState() => _FlipWidgetState();
}

class _FlipWidgetState extends State<FlipWidget> with TickerProviderStateMixin {
  AnimationController? controller;
  late Animation positionAnimation;
  late Animation scaleAnimation;

  ///翻转动画
  AnimationController? animationController;
  late Animation frontAnimation;
  late Animation backAnimation;

  bool isFront = true;
  bool hasHalf = false;

  @override
  void initState() {
    controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 1000))
      ..addStatusListener((status) {
        if (!mounted) return;
        if (status == AnimationStatus.completed) {
          setState(() {});
          Future.delayed(Duration(milliseconds: 500 * (8 - widget.indexLeft)),
              () {
            if (!mounted) return;
            animationController?.forward();
          });
        }
      });
    positionAnimation = RelativeRectTween(
            begin: RelativeRect.fromSize(
                Rect.fromLTWH(
                    ScreenUtil().screenWidth / 2 - ScreenUtil().setWidth(100),
                    ScreenUtil().screenHeight / 3 - ScreenUtil().setWidth(140),
                    ScreenUtil().setWidth(100),
                    ScreenUtil().setWidth(140)),
                Size.zero),
            end: RelativeRect.fromSize(
                Rect.fromLTWH(
                    ScreenUtil().screenWidth / 2 -
                        ScreenUtil().setWidth(100) +
                        (widget.indexX! + 0.5) * ScreenUtil().setWidth(112),
                    ScreenUtil().screenHeight / 3 -
                        ScreenUtil().setWidth(140) +
                        (widget.indexY! + 0.5) * ScreenUtil().setWidth(150),
                    ScreenUtil().setWidth(100),
                    ScreenUtil().setWidth(140)),
                Size.zero))
        .animate(CurvedAnimation(parent: controller!, curve: Curves.linear));
    scaleAnimation = Tween(begin: 0.7, end: 1.0)
        .animate(CurvedAnimation(parent: controller!, curve: Curves.linear));
    animationController = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 600))
      ..addListener(() {
        if (!mounted) return;
        if (animationController!.value > 0.5 && hasHalf == false) {
          isFront = false;
          hasHalf = true;
          setState(() {});
        }
      });
    frontAnimation = Tween(begin: 0.0, end: 0.5).animate(
      CurvedAnimation(
        parent: animationController!,
        curve: const Interval(0.0, 0.5, curve: Curves.easeIn),
      ),
    );
    backAnimation = Tween(begin: 1.5, end: 2.0).animate(
      CurvedAnimation(
        parent: animationController!,
        curve: const Interval(0.5, 1.0, curve: Curves.easeOut),
      ),
    );
    Future.delayed(Duration(milliseconds: 400 * widget.indexLeft), () {
      if (!mounted) return;
      controller?.forward();
    });
    super.initState();
  }

  @override
  void dispose() {
    controller?.dispose();
    animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (controller!.status == AnimationStatus.completed) {
      return Positioned(
        left: ScreenUtil().screenWidth / 2 -
            ScreenUtil().setWidth(100) +
            (widget.indexX! + 0.5) * ScreenUtil().setWidth(112),
        top: ScreenUtil().screenHeight / 3 -
            ScreenUtil().setWidth(140) +
            (widget.indexY! + 0.5) * ScreenUtil().setWidth(150),
        child: AnimatedBuilder(
          animation: hasHalf ? backAnimation : frontAnimation,
          builder: (BuildContext context, Widget? child) {
            var value = (hasHalf ? backAnimation : frontAnimation).value;
            return Transform(
                transform: Matrix4.identity()
                  ..setEntry(3, 2, 0.002)
                  ..rotateY(pi * value),
                alignment: Alignment.center,
                child: child);
          },
          child: IndexedStack(
            index: isFront ? 0 : 1,
            children: <Widget>[widget.frontChild, widget.backChild],
          ),
        ),
      );
    } else {
      return PositionedTransition(
          rect: positionAnimation as Animation<RelativeRect>,
          child: ScaleTransition(
            scale: scaleAnimation as Animation<double>,
            child: AnimatedBuilder(
              animation: hasHalf ? backAnimation : frontAnimation,
              builder: (BuildContext context, Widget? child) {
                var value = (hasHalf ? backAnimation : frontAnimation).value;
                return Transform(
                    transform: Matrix4.identity()
                      ..setEntry(3, 2, 0.002)
                      ..rotateY(pi * value),
                    alignment: Alignment.center,
                    child: child);
              },
              child: IndexedStack(
                index: isFront ? 0 : 1,
                children: <Widget>[widget.frontChild, widget.backChild],
              ),
            ),
          ));
    }
  }
}
