import 'package:flutter/material.dart';

///横向滚动 停在中间
class LeftCenterWidget extends StatefulWidget {
  final Widget child;
  final bool left;

  const LeftCenterWidget({Key? key, required this.child, this.left = true})
      : super(key: key);

  @override
  _LeftCenterWidgetState createState() => _LeftCenterWidgetState();
}

class _LeftCenterWidgetState extends State<LeftCenterWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;

  @override
  void initState() {
    _controller =
        AnimationController(vsync: this, duration: const Duration(seconds: 1));
    _animation = Tween(
            begin:
                widget.left ? const Offset(-1.5, 0.0) : const Offset(1.5, 0.0),
            end: widget.left ? const Offset(-0.4, 0.0) : const Offset(0.4, 0.0))
        .animate(
            CurvedAnimation(parent: _controller!, curve: Curves.easeOutBack));
    Future.delayed(const Duration(milliseconds: 500), () {
      if (!mounted) return;
      _controller!.forward();
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
        position: _animation as Animation<Offset>, child: widget.child);
  }
}
