import 'package:flutter/material.dart';

///横向滚动
class LeftRightWidget extends StatefulWidget {
  final int index;
  final double indexLeft;
  final bool left;
  final Widget child;

  const LeftRightWidget(
      {Key? key,
      required this.child,
      required this.index,
      this.left = true,
      this.indexLeft = 0})
      : super(key: key);

  @override
  _LeftRightWidgetState createState() => _LeftRightWidgetState();
}

class _LeftRightWidgetState extends State<LeftRightWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;

  @override
  void initState() {
    _controller =
        AnimationController(vsync: this, duration: const Duration(seconds: 40));
    _animation = Tween(
            begin:
                widget.left ? const Offset(-2.0, 0.0) : const Offset(5.0, 0.0),
            end: widget.left ? const Offset(5.0, 0.0) : const Offset(-2.0, 0.0))
        .animate(_controller!);
    _controller!.value = widget.index / 6 + widget.indexLeft;
    _controller!.repeat();
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
        position: _animation as Animation<Offset>, child: widget.child);
  }
}
