import 'package:flutter/material.dart';

///数字递增变化
class NumberRunWidget extends StatefulWidget {
  final TextStyle style;
  final TextOverflow? overflow;
  final int? number;

  const NumberRunWidget(
      {Key? key, required this.style, required this.number, this.overflow})
      : super(key: key);

  @override
  _NumberRunWidgetState createState() => _NumberRunWidgetState();
}

class _NumberRunWidgetState extends State<NumberRunWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;
  int? number;

  @override
  void initState() {
    number = widget.number;
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 1000));
    _animation = IntTween(begin: number! < 100 ? 0 : number! - 100, end: number)
        .animate(_controller!);
    super.initState();
  }

  @override
  void didUpdateWidget(NumberRunWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!mounted) return;
    if (number != widget.number) {
      _controller!.reset();
      _animation =
          IntTween(begin: number, end: widget.number).animate(_controller!);
      number = widget.number;
    }
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _controller!.forward();
    return AnimatedBuilder(
        animation: _animation,
        builder: (BuildContext context, Widget? child) {
          return Text('${_animation.value}',
              style: widget.style, overflow: widget.overflow);
        });
  }
}
