import 'dart:math';

import 'package:flutter/cupertino.dart';

class CirclePageRoute extends PageRoute {
  CirclePageRoute({
    required this.builder,
    this.transitionDuration = const Duration(milliseconds: 1000),
    this.opaque = true,
    this.barrierDismissible = false,
    this.barrierColor,
    this.barrierLabel,
    this.maintainState = true,
    RouteSettings? settings,
    bool fullscreenDialog = false,
    this.center,
  }) : super(settings: settings, fullscreenDialog: fullscreenDialog);

  final WidgetBuilder builder;
  final Offset? center;

  @override
  final Duration transitionDuration;

  @override
  final bool opaque;

  @override
  final bool barrierDismissible;

  @override
  final Color? barrierColor;

  @override
  final String? barrierLabel;

  @override
  final bool maintainState;

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    return AnimatedBuilder(
      animation: animation,
      builder: (context, child) {
        return ClipPath(
          clipper: CirclePath(animation.value, center),
          child: child,
        );
      },
      child: builder(context),
    );
  }
}

class CirclePath extends CustomClipper<Path> {
  CirclePath(this.value, this.center);

  final double value;
  final Offset? center;

  @override
  Path getClip(Size size) {
    var path = Path();
    double radius =
        value * sqrt(size.height * size.height + size.width * size.width);
    // AppConfig.printLog(radius);
    //右上角
    // path.addOval(Rect.fromLTRB(
    //     size.width - radius, -radius, size.width + radius, radius));
    //右边
    // path.addOval(Rect.fromLTRB(
    //     radius, size.width + radius, -radius, size.width - radius));
    //
    path.addOval(Rect.fromCenter(
        center: center ?? Offset(size.width / 2, size.height),
        width: radius * 2,
        height: radius * 2));
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return true;
  }
}
