import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

///右边出现停在左边指定位置
class RightLeftWidget extends StatefulWidget {
  final Widget child;
  final int? position;
  final bool anim;

  const RightLeftWidget(
      {Key? key, required this.child, this.position, this.anim = true})
      : super(key: key);

  @override
  _RightLeftWidgetState createState() => _RightLeftWidgetState();
}

class _RightLeftWidgetState extends State<RightLeftWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;
  late Animation _animationFade;
  int? position;

  @override
  void initState() {
    position = widget.position;
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 500))
      ..addStatusListener((status) {
        if (!mounted) return;
        if (status == AnimationStatus.completed) {
          Future.delayed(const Duration(milliseconds: 1500), () {
            if (!mounted) return;
            if (position != 0) {
              _controller?.reset();
              position = position! - 1;
              _controller?.forward();
            } else {
              _controller?.dispose();
              _controller = null;
            }
          });
        }
      });
    _animation =
        Tween(begin: ScreenUtil().setWidth(50), end: 0).animate(_controller!);
    _animationFade = Tween(begin: 1.0, end: 0.0).animate(_controller!);
    if (widget.anim) _controller!.forward();
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: _animation,
        builder: (BuildContext context, Widget? child) {
          double? value = _animation.value +
              (position! - 1) * ScreenUtil().setWidth(50) +
              ScreenUtil().setWidth(32);
          return Positioned(
              left: value,
              width: ScreenUtil().setWidth(60),
              height: ScreenUtil().setWidth(60),
              child: position != 0
                  ? child!
                  : AnimatedBuilder(
                      animation: _animationFade,
                      builder: (BuildContext context, Widget? child) {
                        return Opacity(
                            opacity: _animationFade.value, child: child);
                      },
                      child: child));
        },
        child: widget.child);
  }
}
