import 'package:flutter/material.dart';

///缩放动画
class ScaleWidget extends StatefulWidget {
  final Widget child;
  final int duration;
  final double begin;
  final double end;
  final int stopDuration;

  const ScaleWidget(
      {Key? key,
      required this.child,
      this.duration = 500,
      this.begin = 0.5,
      this.end = 1.0,
      this.stopDuration = 0})
      : super(key: key);

  @override
  _ScaleWidgetState createState() => _ScaleWidgetState();
}

class _ScaleWidgetState extends State<ScaleWidget>
    with SingleTickerProviderStateMixin {
  late AnimationController _controller;
  late Animation<double> _animationScale;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: Duration(milliseconds: widget.duration));
    _animationScale = Tween(begin: widget.begin, end: widget.end).animate(
        CurvedAnimation(parent: _controller, curve: Curves.easeOutBack));
    _controller.repeat(reverse: true);
    if (widget.stopDuration > 0) {
      Future.delayed(Duration(milliseconds: widget.stopDuration), () {
        if (!mounted) {
          return;
        }
        _controller.forward();
      });
    }
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return ScaleTransition(
      scale: _animationScale,
      child: widget.child,
    );
  }
}
