import 'package:flutter/material.dart';

///冒泡动画
class SlideScaleWidget extends StatefulWidget {
  final Widget child;
  final int duration;
  final int left; //0-left 1-top 2-right 3-bottom
  final Curve? curves;

  const SlideScaleWidget(
      {Key? key,
      required this.child,
      this.duration = 500,
      this.left = 0,
      this.curves})
      : super(key: key);

  @override
  _SlideScaleWidgetState createState() => _SlideScaleWidgetState();
}

class _SlideScaleWidgetState extends State<SlideScaleWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;
  late Animation _animationScale;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: Duration(milliseconds: widget.duration));
    switch (widget.left) {
      case 0:
        _animation =
            Tween(begin: const Offset(-1, 0), end: const Offset(0.0, 0.0))
                .animate(CurvedAnimation(
                    parent: _controller!,
                    curve: widget.curves ?? Curves.easeOutBack));
        break;
      case 1:
        _animation =
            Tween(begin: const Offset(0.0, 1), end: const Offset(0.0, 0.0))
                .animate(CurvedAnimation(
                    parent: _controller!,
                    curve: widget.curves ?? Curves.easeOutBack));
        break;
      case 2:
        _animation =
            Tween(begin: const Offset(1, 0), end: const Offset(0.0, 0.0))
                .animate(CurvedAnimation(
                    parent: _controller!,
                    curve: widget.curves ?? Curves.easeOutBack));
        break;
      case 3:
        _animation =
            Tween(begin: const Offset(0.0, -1), end: const Offset(0.0, 0.0))
                .animate(CurvedAnimation(
                    parent: _controller!,
                    curve: widget.curves ?? Curves.easeOutBack));
        break;
      default:
        break;
    }
    _animationScale = Tween(begin: 0.0, end: 1.0).animate(CurvedAnimation(
        parent: _controller!, curve: widget.curves ?? Curves.easeOutBack));
    _controller?.forward();
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
        position: _animation as Animation<Offset>,
        child: ScaleTransition(
          scale: _animationScale as Animation<double>,
          child: widget.child,
        ));
  }
}
