import 'package:flutter/material.dart';

///旋转
class SpinWidget extends StatefulWidget {
  final Widget? child;
  final int seconds;
  final int delayed;
  final bool start;

  const SpinWidget(
      {Key? key,
      this.child,
      this.start = true,
      this.seconds = 5,
      this.delayed = 500})
      : super(key: key);

  @override
  SpinWidgetState createState() => SpinWidgetState();
}

class SpinWidgetState extends State<SpinWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animationRota;

  @override
  void initState() {
    _controller = AnimationController(
        vsync: this, duration: Duration(seconds: widget.seconds));
    _animationRota = Tween(begin: 0.0, end: 1.0)
        .animate(CurvedAnimation(parent: _controller!, curve: Curves.linear));
    if (widget.start) {
      Future.delayed(Duration(milliseconds: widget.delayed), () {
        if (!mounted) return;
        _controller?.repeat();
      });
    }
    super.initState();
  }

  void start() {
    if (!mounted) return;
    _controller?.repeat();
  }

  @override
  void dispose() {
    _controller?.dispose();
    _controller = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return RotationTransition(
        turns: _animationRota as Animation<double>, child: widget.child);
  }
}
