import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:mmoo_base/config/app_config.dart';

class SpreadBtnWidget extends StatefulWidget {
  const SpreadBtnWidget({
    Key? key,
    required this.child,
    required this.radius,
    required this.maxRadius,
    required this.width,
    required this.height,
    this.spreadColor1 = Colors.grey,
    this.spreadColor2 = Colors.grey,
    this.duration = const Duration(milliseconds: 3000),
  }) : super(key: key);

  final Widget child;
  final double radius;
  final double maxRadius;
  final double width;
  final double height;
  final Color spreadColor1;
  final Color spreadColor2;
  final Duration duration;

  @override
  _SpreadBtnWidgetState createState() => _SpreadBtnWidgetState();
}

class _SpreadBtnWidgetState extends State<SpreadBtnWidget>
    with TickerProviderStateMixin {
  List<Widget> children = [];
  List<AnimationController>? controllers = [];
  bool load = false;
  bool starting = true;

  @override
  void initState() {
    children.add(widget.child);
    super.initState();
  }

  start() async {
    while (starting) {
      try {
        if (!mounted) return;
        if (ModalRoute.of(context)?.isCurrent == true) {
          if (!mounted || controllers == null) return;
          AnimationController _animationController;
          Animation<double> _animation;
          _animationController =
              AnimationController(vsync: this, duration: widget.duration);
          _animation = CurvedAnimation(
              parent: _animationController, curve: Curves.linear);
          _animationController.addStatusListener((status) {
            if (!mounted) return;
            if (status == AnimationStatus.completed) {
              _animationController.dispose();
              children.removeAt(0);
              controllers!.removeAt(0);
            }
          });
          controllers!.add(_animationController);
          _animationController.forward();
          children.add(AnimatedSpread(
              animation: _animation,
              radius: widget.radius,
              maxRadius: widget.maxRadius,
              width: widget.width,
              height: widget.height,
              color1: widget.spreadColor1,
              color2: widget.spreadColor2));
          if (!mounted || controllers == null) return;
          setState(() {});
        }
        await Future.delayed(
            Duration(milliseconds: widget.duration.inMilliseconds ~/ 2));
      } catch (e) {
        AppConfig.printLog(e);
      }
    }
  }

  @override
  void dispose() {
    starting = false;
    for (var c in controllers!) {
      try {
        c.dispose();
      } catch (e) {
        AppConfig.printLog(e);
      }
    }
    controllers = null;
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant SpreadBtnWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    children.last = widget.child;
  }

  @override
  Widget build(BuildContext context) {
    if (!load) {
      load = true;
      start();
    }
    return Stack(children: children, alignment: Alignment.center);
  }
}

class AnimatedSpread extends AnimatedWidget {
  final Tween<double> _opacityTween = Tween(begin: 1, end: 0);
  final Tween<double> _radiusTween;
  final Color color1;
  final Color color2;
  final double width;
  final double height;

  AnimatedSpread(
      {Key? key,
      required Animation<double> animation,
      required double radius,
      required double maxRadius,
      required this.width,
      required this.height,
      required this.color1,
      required this.color2})
      : _radiusTween = Tween(begin: radius, end: maxRadius),
        super(key: key, listenable: animation);

  @override
  Widget build(BuildContext context) {
    final Animation<double> animation = listenable as Animation<double>;
    return ClipRRect(
        borderRadius: BorderRadius.circular(
          ScreenUtil().setWidth(50),
        ),
        child: SizedBox(
          width: width + _radiusTween.evaluate(animation),
          height: height + _radiusTween.evaluate(animation),
          child: Opacity(
            opacity: _opacityTween.evaluate(animation),
            child: Container(
              decoration: BoxDecoration(
                gradient: LinearGradient(
                  colors: [
                    color1,
                    color2,
                  ],
                ),
              ),
            ),
          ),
        ));
  }
}
