import 'package:flutter/material.dart';
import 'package:mmoo_base/config/app_config.dart';

class SpreadWidget extends StatefulWidget {
  const SpreadWidget({
    Key? key,
    this.child,
    required this.radius,
    required this.maxRadius,
    this.spreadColor = Colors.grey,
    this.spreadColor2 = Colors.grey,
    this.duration = const Duration(milliseconds: 3000),
  }) : super(key: key);

  final Widget? child;
  final double? radius;
  final double? maxRadius;
  final Color? spreadColor;
  final Color? spreadColor2;
  final Duration duration;

  @override
  _SpreadWidgetState createState() => _SpreadWidgetState();
}

class _SpreadWidgetState extends State<SpreadWidget>
    with TickerProviderStateMixin {
  List<Widget> children = [];
  List<AnimationController>? controllers = [];
  bool load = false;
  bool starting = true;

  @override
  void initState() {
    if (widget.child != null) {
      children.add(
        ClipOval(
          child: SizedBox(
              width: widget.radius, height: widget.radius, child: widget.child),
        ),
      );
    }
    super.initState();
  }

  start() async {
    while (starting) {
      try {
        if (!mounted) return;
        // if (ModalRoute.of(context)?.isCurrent == true) {
        if (!mounted || controllers == null) return;
        AnimationController _animationController;
        Animation<double> _animation;
        _animationController =
            AnimationController(vsync: this, duration: widget.duration);
        _animation =
            CurvedAnimation(parent: _animationController, curve: Curves.linear);
        _animationController.addStatusListener((status) {
          if (!mounted) return;
          if (status == AnimationStatus.completed) {
            children.removeAt(0);
            controllers!.removeAt(0);
            _animationController.dispose();
          }
        });
        controllers!.add(_animationController);
        _animationController.forward();
        widget.child != null
            ? children.insert(
                children.length - 1,
                AnimatedSpread(
                    animation: _animation,
                    radius: widget.radius,
                    maxRadius: widget.maxRadius,
                    color: widget.spreadColor,
                    color2: widget.spreadColor2))
            : children.add(AnimatedSpread(
                animation: _animation,
                radius: widget.radius,
                maxRadius: widget.maxRadius,
                color: widget.spreadColor,
                color2: widget.spreadColor2));
        setState(() {});
        // }
        await Future.delayed(
            Duration(milliseconds: widget.duration.inMilliseconds ~/ 3));
      } catch (e) {
        AppConfig.printLog(e);
      }
    }
  }

  @override
  void dispose() {
    starting = false;
    for (var c in controllers!) {
      c.dispose();
    }
    controllers = null;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!load) {
      load = true;
      start();
    }
    return Stack(children: children, alignment: Alignment.center);
  }
}

class AnimatedSpread extends AnimatedWidget {
  final Tween<double> _opacityTween = Tween(begin: 1, end: 0);
  final Tween<double> _radiusTween;
  final Color? color;
  final Color? color2;

  AnimatedSpread(
      {Key? key,
      required Animation<double> animation,
      required double? radius,
      required double? maxRadius,
      required this.color,
      required this.color2})
      : _radiusTween = Tween(begin: radius, end: maxRadius),
        super(key: key, listenable: animation);

  @override
  Widget build(BuildContext context) {
    final Animation<double> animation = listenable as Animation<double>;
    return SizedBox(
      width: _radiusTween.evaluate(animation),
      height: _radiusTween.evaluate(animation),
      child: ClipOval(
        child: Opacity(
            opacity: _opacityTween.evaluate(animation),
            child: Container(
              decoration: BoxDecoration(
                gradient: LinearGradient(
                  begin: Alignment.topCenter,
                  end: Alignment.bottomCenter,
                  colors: [color!, color2 ?? color!],
                ),
              ),
            )),
      ),
    );
  }
}
