import 'dart:typed_data';

import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:mmoo_base/utils/download_utils.dart';
import 'package:svgaplayer_flutter/parser.dart';
import 'package:svgaplayer_flutter/player.dart';

class SvgaDownloadWidget extends StatefulWidget {
  final double widht;
  final String? iconUrl;
  final String svgaurl;
  final String? headImage;
  double? height;

  SvgaDownloadWidget(this.svgaurl,
      {required this.widht, this.iconUrl, this.height, this.headImage});

  @override
  SvgaDownloadWidgetState createState() => SvgaDownloadWidgetState();
}

class SvgaDownloadWidgetState extends State<SvgaDownloadWidget>
    with SingleTickerProviderStateMixin {
  late SVGAAnimationController animationController;
  double headWidth = 0;
  static String _svgaurl = '';

  @override
  Widget build(BuildContext context) {
    if (widget.height == null) {
      widget.height = widget.widht;
    }
    return Container(
      width: widget.widht,
      height: widget.height,
      child: Stack(
        children: [
          Visibility(
              visible: widget.headImage != null,
              child: Positioned.fill(
                  child: Container(
                alignment: Alignment.center,
                width: headWidth,
                height: headWidth,
                child: ClipRRect(
                  borderRadius: BorderRadius.circular(headWidth * 0.5),
                  child: ExtendedImage.network(
                      compressImageUrl(widget.headImage ?? ''),
                      alignment: Alignment.center,
                      fit: BoxFit.cover,
                      enableSlideOutPage: true,
                      mode: ExtendedImageMode.gesture,
                      enableLoadState: true,
                      handleLoadingProgress: true,
                      width: headWidth,
                      height: headWidth),
                  // child: BLCommonUtils.getLoadAssetWidget(widget.headImage,
                  //     urlWidth: BLAppConfig.PIC_WIDTH_HEAD,
                  //     alignment: Alignment.topCenter,
                  //     width: widget.widht * (5 / 7.0),
                  //     height: widget.widht * (5 / 7.0)),
                ),
              ))),
          Positioned.fill(
              child: (widget.svgaurl.isNotEmpty &&
                      this.animationController.videoItem != null)
                  ? SVGAImage(this.animationController)
                  : (widget.iconUrl != null
                      ? ExtendedImage.network(
                          compressImageUrl(widget.iconUrl ?? ''),
                          alignment: Alignment.center,
                          fit: BoxFit.cover,
                          enableSlideOutPage: true,
                          mode: ExtendedImageMode.gesture,
                          enableLoadState: true,
                          handleLoadingProgress: true,
                          width: widget.widht,
                          height: widget.widht,
                          // child: BLCommonUtils.getLoadAssetWidget(widget.headImage,
                          //     urlWidth: BLAppConfig.PIC_WIDTH_HEAD,
                          //     alignment: Alignment.topCenter,
                          //     width: widget.widht * (5 / 7.0),
                          //     height: widget.widht * (5 / 7.0)),
                        )
                      // BLCommonUtils.getLoadAssetWidget(
                      //     widget.iconUrl,
                      //     urlWidth: BLAppConfig.PIC_WIDTH_HEAD,
                      //     alignment: Alignment.topCenter,
                      //   )
                      : Container())),
        ],
      ),
    );
  }

  void loadAnimation() async {
    if (widget.svgaurl.isNotEmpty) {
      Uint8List? uint8list =
          await DownloadUtils.downloadSvgaAndSave(widget.svgaurl);
      if (uint8list == null) {
        return;
      }
      final videoItem = await SVGAParser.shared.decodeFromBuffer(uint8list);
      this.animationController.videoItem = videoItem;
      this.animationController.repeat().whenComplete(() {
        this.animationController.videoItem = null;
      });
      if (mounted) {
        setState(() {});
      }
    }
  }

  String compressImageUrl(String url) {
    try {
      if (url.startsWith('http')) {
        if (url.endsWith('!user')) url = url.replaceAll('!user', '');
      }
      var spacing = url.contains('?') ? '&' : '?';
      return '$url${spacing}x-oss-process=image/resize,w_200';
    } catch (e) {}
    return url;
  }

  @override
  void initState() {
    if (widget.height == null) {
      widget.height = widget.widht;
    }

    if (widget.svgaurl.isEmpty && (widget.iconUrl ?? '').isEmpty) {
      headWidth = widget.widht;
    } else {
      headWidth = widget.widht * (5 / 7.0);
    }
    this.animationController = SVGAAnimationController(vsync: this);
    this.loadAnimation();
    super.initState();
  }

  @override
  void dispose() {
    this.animationController.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(SvgaDownloadWidget oldWidget) {
    if (_svgaurl != widget.svgaurl) {
      _svgaurl = widget.svgaurl;
      loadAnimation();
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }
}
