import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:svgaplayer_flutter/parser.dart';
import 'package:svgaplayer_flutter/player.dart';

///svga封装
class SVGAPlayerWidget extends StatefulWidget {
  final String? resUrl;
  final String? assetsName;
  final File? file;

  SVGAPlayerWidget({Key? key, this.resUrl, this.assetsName, this.file})
      : super(key: key);

  @override
  State<SVGAPlayerWidget> createState() {
    return _SVGAPlayerWidgetState();
  }
}

class _SVGAPlayerWidgetState extends State<SVGAPlayerWidget>
    with SingleTickerProviderStateMixin {
  SVGAAnimationController? animationController;

  @override
  void initState() {
    this.animationController = SVGAAnimationController(vsync: this);
    Future? decode;
    if (widget.resUrl != null)
      decode = SVGAParser.shared.decodeFromURL(widget.resUrl!);
    else if (widget.assetsName != null)
      decode =
          SVGAParser.shared.decodeFromAssets('assets/' + widget.assetsName!);
    else if (widget.file != null)
      decode = widget.file!.readAsBytes().then((bytes) {
        return SVGAParser.shared.decodeFromBuffer(bytes);
      });
    if (decode != null)
      decode.then((videoItem) {
        if (mounted) {
          this.animationController
            ?..videoItem = videoItem
            ..forward().whenComplete(() {
              if (!mounted) return;
              this.animationController?.clear();
              this.animationController?.videoItem = null;
              Navigator.pop(context);
            });
        }
      });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SVGAImage(this.animationController!, fit: BoxFit.cover);
  }

  @override
  void dispose() {
    this.animationController?.videoItem = null;
    this.animationController?.dispose();
    super.dispose();
  }
}
