import 'package:flutter/material.dart';

import 'spread_widget.dart';

///缩放动画，大-小无限循环
class ZoomWidget extends StatefulWidget {
  final Widget child;
  final double? size;
  final double? maxSize;
  final Color? color;
  final Color? color2;

  const ZoomWidget(
      {Key? key,
      required this.child,
      this.size,
      this.maxSize,
      this.color,
      this.color2})
      : super(key: key);

  @override
  _ZoomWidgetState createState() => _ZoomWidgetState();
}

class _ZoomWidgetState extends State<ZoomWidget>
    with SingleTickerProviderStateMixin {
  AnimationController? _controller;
  late Animation _animation;

  @override
  void initState() {
    _controller =
        AnimationController(vsync: this, duration: const Duration(seconds: 1));
    _animation = Tween(begin: 0.5, end: 1.0).animate(
        CurvedAnimation(parent: _controller!, curve: Curves.easeOutBack));
    _controller!.repeat(reverse: true);
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: Alignment.center,
      children: <Widget>[
        SpreadWidget(
            child: Container(
              decoration: BoxDecoration(
                gradient: LinearGradient(
                  begin: Alignment.topCenter,
                  end: Alignment.bottomCenter,
                  colors: [widget.color!, widget.color2 ?? widget.color!],
                ),
              ),
            ),
            radius: widget.size,
            maxRadius: widget.maxSize,
            spreadColor: widget.color,
            spreadColor2: widget.color2),
        ScaleTransition(
            scale: _animation as Animation<double>, child: widget.child)
      ],
    );
  }
}
