import 'dart:async';
import 'dart:io';

import 'package:android_intent/android_intent.dart';
import 'package:connectivity/connectivity.dart';
import 'package:date_format/date_format.dart' as date_format;
import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_cache_manager/flutter_cache_manager.dart';
import 'package:mmoo_base/api/result_data.dart';
import 'package:mmoo_base/config/app_config.dart';
import 'package:mmoo_base/generated/l10n.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';

///通用逻辑-基本方法管理
class CommonUtils {
  static int pageStatusSuccess = 0;
  static int pageStatusLoading = 1;
  static int pageStatusError = 2;
  static int pageStatusEmpty = 3;
  static int pageStatusProgress = 4;
  static dynamic toast;

  static void showToast(msg) {
    AppConfig.printLog('showToast: $msg');
    toast?.call(msg);
  }

  static String getTimeStr(DateTime date, {bool min = false}) {
    if (min) {
      return date_format
          .formatDate(date, [date_format.nn, ':', date_format.ss]);
    } else {
      return date_format.formatDate(
          date, [date_format.HH, ':', date_format.nn, ':', date_format.ss]);
    }
  }

  static String getTimeDayStr(String time, {bool min = true}) {
    if (time.isEmpty == false) {
      if (min) {
        return date_format
            .formatDate(DateTime.fromMillisecondsSinceEpoch(int.parse(time)), [
          date_format.yyyy,
          '-',
          date_format.mm,
          '-',
          date_format.dd,
          ' ',
          date_format.HH,
          ':',
          date_format.nn,
          ':',
          date_format.ss
        ]);
      } else {
        return date_format.formatDate(
            DateTime.fromMillisecondsSinceEpoch(int.parse(time)),
            [date_format.yyyy, '-', date_format.mm, '-', date_format.dd]);
      }
    }
    return '';
  }

  static const double millisLimit = 1000.0;

  static const double secondsLimit = 60 * millisLimit;

  static const double minutesLimit = 60 * secondsLimit;

  static const double hoursLimit = 24 * minutesLimit;

  static const double daysLimit = 30 * hoursLimit;

  ///日期格式转换
  static String getNewsTimeStr(context, DateTime date) {
    int subTimes =
        DateTime.now().millisecondsSinceEpoch - date.millisecondsSinceEpoch;
    if (subTimes < millisLimit) {
      return S.of(context).strTimeNow;
    } else if (subTimes < secondsLimit) {
      return S
          .of(context)
          .strTimeSeconds((subTimes / millisLimit).round().toString());
    } else if (subTimes < minutesLimit) {
      return S
          .of(context)
          .strTimeMin((subTimes / secondsLimit).round().toString());
    } else if (subTimes < hoursLimit) {
      return S
          .of(context)
          .strTimeHours((subTimes / minutesLimit).round().toString());
    } else if (subTimes < daysLimit) {
      return S
          .of(context)
          .strTimeDays((subTimes / hoursLimit).round().toString());
    } else {
      return date_format.formatDate(
          date, [date_format.yyyy, '-', date_format.mm, '-', date_format.dd]);
    }
  }

  ///是否需要显示时间，相差2分钟
  static bool needShowTime(int sentTime1, int sentTime2) {
    return (sentTime1 - sentTime2).abs() > 2 * 60 * 1000;
  }

  static getLocalPath() async {
    Directory? appDir;
    if (Platform.isIOS) {
      appDir = await getApplicationDocumentsDirectory();
    } else {
      appDir = await getExternalStorageDirectory();
      PermissionStatus status = await Permission.storage.status;
      if (status == PermissionStatus.denied) {
        status = await Permission.storage.request();
        if (status == PermissionStatus.denied) return null;
      }
    }
    String appDocPath = appDir!.path + '/SaveImageTemp';
    Directory appPath = Directory(appDocPath);
    await appPath.create(recursive: true);
    return appPath;
  }

  static saveImage(String url) async {
    Future<String?> _findPath(String imageUrl) async {
      final file = await DefaultCacheManager().getSingleFile(url);
      Directory localPath = await getLocalPath();
      final name = splitFileNameByPath(file.path);
      final result = await file.copy(localPath.path + name);
      return result.path;
    }

    return _findPath(url);
  }

  static splitFileNameByPath(String path) {
    return path.substring(path.lastIndexOf('/'));
  }

  static String compressImageUrl(String url, {int width = 0}) {
    try {
      if (url.startsWith('http') && width != 0) {
        if (url.endsWith('!user')) url = url.replaceAll('!user', '');
      }
      var spacing = url.contains('?') ? '&' : '?';
      return '$url${spacing}x-oss-process=image/resize,w_${width * 3}';
    } catch (e) {
      AppConfig.printLog(e);
    }
    return url;
  }

  /// 不退出
  static Future<bool> dialogExitApp(BuildContext context) async {
    ///如果是 android 回到桌面
    if (Platform.isAndroid) {
      AndroidIntent intent = const AndroidIntent(
          action: 'android.intent.action.MAIN',
          category: 'android.intent.category.HOME');
      await intent.launch();
    }
    return Future.value(false);
  }

  static String compareString(String? str) {
    return str?.length == 1 ? '0$str' : (str ?? '');
  }

  static bool isReception = true;

  static checkIsCurrent(context, back, {noBack}) async {
    return await Future(() {
      try {
        if (isReception && ModalRoute.of(context)?.isCurrent == true) {
          return back();
        } else {
          return noBack?.call();
        }
      } catch (e) {
        AppConfig.printLog(e);
      }
    });
  }

  /// 错误提示统一
  static showErrorToast(BuildContext? context, {res, back}) async {
    try {
      if (context == null) {
        return;
      }
      var connectivityResult = await (Connectivity().checkConnectivity());
      if (connectivityResult == ConnectivityResult.none) {
        back?.call(S.of(context).strNetworkError);
      } else {
        if (res == null) {
          back?.call(S.of(context).strAppHttpErr);
        } else if (res is String) {
          back?.call(res);
        } else if (res.data is! ResultData) {
          if (res.dataError == true) {
            back?.call(S.of(context).strAppHttpErr);
          } else {
            try {
              if (res.data?.msg == null) {
                back?.call(S.of(context).strAppHttpErr);
              } else {
                back?.call(res.data?.msg);
              }
            } catch (e) {
              back?.call(S.of(context).strAppHttpErr);
            }
          }
        } else {
          back?.call(S.of(context).strAppHttpErr);
        }
      }
    } catch (e) {
      AppConfig.printLog(e);
    }
  }

  static dynamic toastLoad;
  static dynamic toastDismiss;

  static void showLoading({String? msg}) {
    AppConfig.printLog('showToast: $msg');
    toastLoad?.call(msg ?? '');
  }

  static void dismiss() {
    toastDismiss?.call();
  }

  static Widget getLoadWidget(
    String? url,
    String imageIcon,
    String imageUrl,
    String load,
    Color colorBg,
    Color colorProgress, {
    double? width,
    double? height,
    Alignment? alignment,
    BoxFit? fit,
    String? package,
    Map<String, String>? headers,
  }) {
    if (url?.isEmpty == false) {
      if (url!.startsWith(imageIcon)) {
        return Image.asset(
          url,
          width: width,
          height: height,
          fit: fit,
          package: package,
        );
      } else if (!url.startsWith('http')) {
        return ExtendedImage.file(
          File(url),
          width: width,
          height: height,
          fit: fit ?? BoxFit.cover,
          alignment: alignment ?? Alignment.topCenter,
          enableLoadState: true,
          loadStateChanged: ((ExtendedImageState state) {
            switch (state.extendedImageLoadState) {
              case LoadState.loading:
                if (state.loadingProgress == null) {
                  return Image.asset(
                    load,
                    width: width,
                    height: height,
                    fit: fit ?? BoxFit.cover,
                    alignment: alignment ?? Alignment.center,
                    package: package,
                  );
                }
                return Container(
                  color: colorBg,
                  child: Center(
                    child: CircularProgressIndicator(
                        valueColor:
                            AlwaysStoppedAnimation<Color>(colorProgress),
                        value: state.loadingProgress!.expectedTotalBytes != null
                            ? state.loadingProgress!.cumulativeBytesLoaded /
                                state.loadingProgress!.expectedTotalBytes!
                            : null),
                  ),
                );
              case LoadState.completed:
                return state.completedWidget;
              default:
                return Image.asset(
                  load,
                  width: width,
                  height: height,
                  fit: fit ?? BoxFit.cover,
                  alignment: alignment ?? Alignment.center,
                  package: package,
                );
            }
          }),
        );
      } else {
        return ExtendedImage.network(
          !url.startsWith(imageUrl) && headers == null && (width ?? 0) != 0
              ? compressImageUrl(url, width: width!.toInt())
              : url,
          width: width,
          height: height,
          fit:
              fit ?? (url.startsWith(imageUrl) ? BoxFit.contain : BoxFit.cover),
          headers: headers,
          alignment: alignment ?? Alignment.topCenter,
          enableLoadState: true,
          handleLoadingProgress: true,
          loadStateChanged: ((ExtendedImageState state) {
            switch (state.extendedImageLoadState) {
              case LoadState.loading:
                if (state.loadingProgress == null) {
                  return Image.asset(
                    load,
                    width: width,
                    height: height,
                    fit: fit ?? BoxFit.cover,
                    alignment: alignment ?? Alignment.center,
                    package: package,
                  );
                }
                return Container(
                  color: colorBg,
                  child: Center(
                    child: CircularProgressIndicator(
                        valueColor:
                            AlwaysStoppedAnimation<Color>(colorProgress),
                        value: state.loadingProgress!.expectedTotalBytes != null
                            ? state.loadingProgress!.cumulativeBytesLoaded /
                                state.loadingProgress!.expectedTotalBytes!
                            : null),
                  ),
                );
              case LoadState.completed:
                return state.completedWidget;
              default:
                return Image.asset(
                  load,
                  width: width,
                  height: height,
                  fit: fit ?? BoxFit.cover,
                  alignment: alignment ?? Alignment.center,
                  package: package,
                );
            }
          }),
        );
      }
    } else {
      return Image.asset(
        load,
        width: width,
        height: height,
        fit: fit ?? BoxFit.cover,
        alignment: alignment ?? Alignment.center,
        package: package,
      );
    }
  }
}
