import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:mmoo_base/ui/other/circle_route.dart';

///导航类
class NavigatorUtils {
  static Map routeMap = {};
  static VoidCallback? goGuide;

  ///公共打开方式
  static Future navigatorPush(BuildContext context, Widget widget, String name,
      {bool add = false,
      bool replace = false,
      bool fullscreenDialog = false,
      bool match = false,
      Offset? center}) {
    FocusScope.of(context).unfocus();
    var route = match
        ? CirclePageRoute(
            fullscreenDialog: fullscreenDialog,
            settings: RouteSettings(name: name),
            center: center,
            builder: (context) => widget)
        : CupertinoPageRoute(
            fullscreenDialog: fullscreenDialog,
            settings: RouteSettings(name: name),
            builder: (context) => widget);
    if (add) {
      if (routeMap[name]?.isActive == true) {
        Navigator.removeRoute(context, routeMap[name]);
      }
      routeMap[name] = route;
    }
    if (replace) {
      return Navigator.pushReplacement(context, route);
    } else {
      return Navigator.push(context, route);
    }
  }

  static popUntil(BuildContext context, String name) {
    FocusScope.of(context).unfocus();
    Navigator.popUntil(context, ModalRoute.withName(name));
  }

  ///Page页面的容器，做一次通用自定义
  static Widget pageContainer(context, widget) {
    return MediaQuery(

        ///不受系统字体缩放影响
        data: MediaQuery.of(context).copyWith(textScaleFactor: 1.0),
        child: widget);
  }

  static goGuide2() {
    goGuide?.call();
  }

  ///弹出 dialog
  static Future<T?> showBaseDialog<T>(
      {required BuildContext context,
      bool dismiss = false,
      WidgetBuilder? builder,
      Color color = Colors.black54,
      String? name}) {
    FocusScope.of(context).unfocus();
    return showDialog<T>(
        context: context,
        barrierColor: color,
        routeSettings: RouteSettings(name: name ?? 'showBaseDialog'),
        builder: (context) {
          return GestureDetector(
            behavior: HitTestBehavior.opaque,
            onTap: () {
              if (dismiss) {
                Navigator.pop(context);
              }
            },
            child: Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor: Colors.transparent,
              body: MediaQuery(
                ///不受系统字体缩放影响
                data: MediaQuery.of(context).copyWith(textScaleFactor: 1.0),
                child: builder!(context),
              ),
            ),
          );
        });
  }

  static int clicks = 0;
  static int jumpTime = 0;
  static Function? downFn;

  ///防止多次点击
  static VoidCallback debounce(Function fn, [int t = 500]) {
    return () {
      if (downFn != fn) {
        clicks = 0;
        downFn = fn;
        jumpTime = 0;
      }
      if (jumpTime >= (DateTime.now().millisecondsSinceEpoch - t) &&
          clicks <= 10) {
        clicks++;
        jumpTime = DateTime.now().millisecondsSinceEpoch;
      } else {
        jumpTime = DateTime.now().millisecondsSinceEpoch;
        clicks = 0;
        fn();
      }
    };
  }
}
